﻿using System;
using System.Collections.Generic;

namespace GenericMethods
{
    public class MathGeneric
    {
        public static T Min<T>(T a, T b) where T : IComparable<T>
        {
            if (a.CompareTo(b) < 0) return a;
            return b;
        }

        public static T Max<T>(T a, T b) where T : IComparable<T>
        {
            if (a.CompareTo(b) < 0) return b;
            return a;
        }

        public static T Min<T>(T a, T b, IComparer<T> comparer)
        {
            if (comparer.Compare(a, b) < 0) return a;
            return b;
        }

        public static T Max<T>(T a, T b, IComparer<T> comparer)
        {
            if (comparer.Compare(a, b) < 0) return b;
            return a;
        }

        public static IDictionary<T, int> Frequencies<T>(IEnumerable<T> elements)
        {
            IDictionary<T, int> dictionary = new Dictionary<T, int>();
            foreach (T element in elements)
            {
                if (dictionary.ContainsKey(element))
                {
                    int number = dictionary[element];
                    number++;
                    dictionary[element] = number;
                }
                else
                {
                    dictionary[element] = 1;
                }
            }
            return dictionary;
        }

        public static List<T> MakeList<T>(T element)
        {
            List<T> list = new List<T> { element };
            return list;
        }

        public static ICollection<T> InterSection<T>(ICollection<T> coll1, ICollection<T> coll2)
        {
            ICollection<T> result = new List<T>();
            foreach (T element in coll1)
            {
                if (coll2.Contains(element))
                {
                    result.Add(element);
                }
            }
            return result;
        }
    }
}
